import typer
from pathlib import Path
from typing import Optional

from pldatacli.io.loader import load_lazyframe
from pldatacli.commands.filter import apply_filters
from pldatacli.commands.groupby import apply_groupby
from pldatacli.commands.sort import apply_sort
from pldatacli.commands.rounding import apply_round
from pldatacli.render.table import render_df

app = typer.Typer()


@app.command()
def query(
    file: Path,
    filter: list[str] = typer.Option(None),
    groupby: list[str] = typer.Option(None),
    agg: list[str] = typer.Option(None),
    sort: list[str] = typer.Option(None),
    round_digits: Optional[int] = typer.Option(None, "--round"),
):
    lf = load_lazyframe(file)
    lf = apply_filters(lf, filter)
    lf = apply_groupby(lf, groupby, agg)
    lf = apply_sort(lf, sort)
    lf = apply_round(lf, round_digits)

    df = lf.collect()
    render_df(df)



from pldatacli.parsers.agg_parser import parse_aggs


def apply_groupby(lf, groupby_cols, agg_strings):

    if not groupby_cols:
        return lf

    agg_exprs = parse_aggs(agg_strings)

    return lf.group_by(groupby_cols).agg(agg_exprs)



import polars as pl


AGG_MAP = {
    "sum": lambda c: pl.col(c).sum(),
    "mean": lambda c: pl.col(c).mean(),
    "max": lambda c: pl.col(c).max(),
    "min": lambda c: pl.col(c).min(),
    "std": lambda c: pl.col(c).std(),
    "count": lambda c: pl.col(c).count(),
    "distinct_count": lambda c: pl.col(c).n_unique(),
}


def parse_aggs(agg_strings):

    if not agg_strings:
        return []

    exprs = []

    for a in agg_strings:
        col, ops = a.split("=")

        for op in ops.split(","):
            exprs.append(AGG_MAP[op](col).alias(f"{col}_{op}"))

    return exprs



from rich.console import Console
from rich.table import Table

console = Console()


def render_df(df):

    table = Table()

    for col in df.columns:
        table.add_column(col)

    for row in df.iter_rows():
        table.add_row(*[str(x) for x in row])

    console.print(table)
